
#include "Fluid.h"
#include <cmath>


CFluid::CFluid( int size/*, bool cylinder*/ )
:	/*mCylinder( cylinder ),*/
	mSize( size ),
	mLayerSize( size * size ),
	mTotalSize( size * size * size )
{
	mDeltaTime = 0.4f;
	mViscosity = 0.01f;
	mDamping = 0.001f;
	mTemperatureDissipation = 0.05f;
	
	int sz = mSize * mSize * (mSize+2);
	mVelocityX = new float[ sz ];
	ZeroMemory( mVelocityX, sz * sizeof(float) );
	mVelocityY = new float[ sz ];
	ZeroMemory( mVelocityY, sz * sizeof(float) );
	mVelocityZ = new float[ sz ];
	ZeroMemory( mVelocityZ, sz * sizeof(float) );
	mVelocityX0 = new float[ sz ];
	ZeroMemory( mVelocityX0, sz * sizeof(float) );
	mVelocityY0 = new float[ sz ];
	ZeroMemory( mVelocityY0, sz * sizeof(float) );
	mVelocityZ0 = new float[ sz ];
	ZeroMemory( mVelocityZ0, sz * sizeof(float) );

	/*
	mInside = new int[ mTotalSize ];
	if( mCylinder ) {
		int *ins = mInside;
		for( int x = 0; x < mSize; ++x ) {
			for( int y = 0; y < mSize; ++y ) {
				for( int z = 0; z < mSize; ++z ) {
					int dx = x-mSize/2;
					int dz = z-mSize/2;
					int r2 = dx*dx + dz*dz;
					if( r2 > mSize*mSize/4 )
						*ins = 0;
					else
						*ins = 1;
					++ins;
				}
			}
		}
	} else {
		for( int i = 0; i < mTotalSize; ++i )
			mInside[i] = 1;
	}
	*/

	mTemperature = new float[ mTotalSize ];
	ZeroMemory( mTemperature, mTotalSize * sizeof(float) );
	mTemperature0 = new float[ mTotalSize ];
	ZeroMemory( mTemperature0, mTotalSize * sizeof(float) );
	
	mPlanR2C = rfftw3d_create_plan( mSize, mSize, mSize, FFTW_REAL_TO_COMPLEX, FFTW_IN_PLACE );
	mPlanC2R = rfftw3d_create_plan( mSize, mSize, mSize, FFTW_COMPLEX_TO_REAL, FFTW_IN_PLACE );
}

CFluid::~CFluid()
{
	delete[] mVelocityX;
	delete[] mVelocityY;
	delete[] mVelocityZ;
	delete[] mVelocityX0;
	delete[] mVelocityY0;
	delete[] mVelocityZ0;
	//delete[] mInside;
	delete[] mTemperature;
	delete[] mTemperature0;
	rfftwnd_destroy_plan( mPlanR2C );
	rfftwnd_destroy_plan( mPlanC2R );
}

void CFluid::resetVelocity0()
{
	int sz = mSize * mSize * (mSize+2);
	ZeroMemory( mVelocityX0, sz * sizeof(float) );
	ZeroMemory( mVelocityY0, sz * sizeof(float) );
	ZeroMemory( mVelocityZ0, sz * sizeof(float) );
}


void CFluid::interpolatePrepare( float x, float y, float z ) const
{
	SInterpolatePrepare& p = mInterpolate;
	
	// get grid coords for x,y,z point and fractional parts
	p.i0 = floori( x );
	p.fi = x - p.i0;
	p.i0 = (mSize+(p.i0%mSize))%mSize;
	p.i1 = (p.i0+1)%mSize;
	p.j0 = floori( y );
	p.fj = y - p.j0;
	p.j0 = (mSize+(p.j0%mSize))%mSize;
	p.j1 = (p.j0+1)%mSize;
	p.k0 = floori( z );
	p.fk = z - p.k0;
	p.k0 = (mSize+(p.k0%mSize))%mSize;
	p.k1 = (p.k0+1)%mSize;
	// interpolate indices of 8 corners
	int szj0 = mSize*p.j0;
	int szj1 = mSize*p.j1;
	int szk0 = mSize*mSize*p.k0;
	int szk1 = mSize*mSize*p.k1;
	p.idx[0] = p.i0 + szj0 + szk0;
	p.idx[1] = p.i0 + szj0 + szk1;
	p.idx[2] = p.i0 + szj1 + szk0;
	p.idx[3] = p.i0 + szj1 + szk1;
	p.idx[4] = p.i1 + szj0 + szk0;
	p.idx[5] = p.i1 + szj0 + szk1;
	p.idx[6] = p.i1 + szj1 + szk0;
	p.idx[7] = p.i1 + szj1 + szk1;
}

float CFluid::interpolate( const float* field ) const
{
	SInterpolatePrepare& p = mInterpolate;

	float infi = 1.0f - p.fi;
	float infj = 1.0f - p.fj;
	float infk = 1.0f - p.fk;
	return
		infi * (
		infj * ( infk*field[p.idx[0]] + p.fk*field[p.idx[1]] )+
		p.fj * ( infk*field[p.idx[2]] + p.fk*field[p.idx[3]] ) ) +
		p.fi * (
		infj * ( infk*field[p.idx[4]] + p.fk*field[p.idx[5]] )+
		p.fj * ( infk*field[p.idx[6]] + p.fk*field[p.idx[7]] ) );
}


void CFluid::solveVelocity()
{
	int i, j, k;
	
	//
	// add forces to velocity

	for( i = 0; i < mTotalSize; ++i ) {
		//if( !mInside[i] )
		//	continue;
		mVelocityX[i] += mDeltaTime * mVelocityX0[i];
		mVelocityY[i] += mDeltaTime * mVelocityY0[i];
		mVelocityZ[i] += mDeltaTime * mVelocityZ0[i];
		mVelocityX0[i] = mVelocityX[i];
		mVelocityY0[i] = mVelocityY[i];
		mVelocityZ0[i] = mVelocityZ[i];
	}
	
	//
	// self-advection (diffuse velocities)

	float x, y, z;
	float invSize = 1.0f / mSize;
	float invHalfSize = invSize * 0.5f;
	for( i = 0, x = invHalfSize; i < mSize; ++i, x += invSize ) {
		for( j = 0, y = invHalfSize; j < mSize; ++j, y += invSize ) {
			for( k = 0, z = invHalfSize; k < mSize; ++k, z += invSize ) {
				int idxIJK = i + mSize*j + mSize*mSize*k;
				float x0 = mSize * ( x - mDeltaTime * mVelocityX0[ idxIJK ] ) - 0.5f;
				float y0 = mSize * ( y - mDeltaTime * mVelocityY0[ idxIJK ] ) - 0.5f;
				float z0 = mSize * ( z - mDeltaTime * mVelocityZ0[ idxIJK ] ) - 0.5f;
				interpolatePrepare( x0, y0, z0 );
				mVelocityX[idxIJK] = interpolate( mVelocityX0 );
				mVelocityY[idxIJK] = interpolate( mVelocityY0 );
				mVelocityZ[idxIJK] = interpolate( mVelocityZ0 );
			}
		}
	}

	//
	// viscosity and mass-conservation

	for( i = 0; i < mSize; ++i ) {
		for( j = 0; j < mSize; ++j ) {
			for( k = 0; k < mSize; ++k ) {
				int idx = i + mSize*j + mSize*mSize*k;
				int idx0 = i + (mSize+2)*j + mSize*(mSize+2)*k;
				mVelocityX0[idx0] = mVelocityX[idx];
				mVelocityY0[idx0] = mVelocityY[idx];
				mVelocityZ0[idx0] = mVelocityZ[idx];
			}
		}
	}
	// transform into fourier domain
	fftR2C( mVelocityX0 );
	fftR2C( mVelocityY0 );
	fftR2C( mVelocityZ0 );
	
	// filter & conserve mass
	for( i = 0 ; i <= mSize ; i += 2 ) {
		x = 0.5f * i;
		for ( j = 0 ; j < mSize ; ++j ) {
			y = (j <= mSize/2) ? (float)j : (float)(j-mSize);
			for ( k = 0 ; k < mSize; ++k ) {
				z = (k <= mSize/2) ? (float)k : (float)(k-mSize);
				float r = x*x + y*y + z*z;
				if( r == 0.0f )
					continue;
				// filter amount
				float f = (float)exp( -r * mDeltaTime * mViscosity );

				// get complex numbers
				int idxR = i + (mSize+2)*j + (mSize+2)*mSize*k;
				int idxI = idxR + 1;
				float cxR = mVelocityX0[ idxR ], cxI = mVelocityX0[ idxI ];
				float cyR = mVelocityY0[ idxR ], cyI = mVelocityY0[ idxI ];
				float czR = mVelocityZ0[ idxR ], czI = mVelocityZ0[ idxI ];

				// conserve mass (project onto plane with normal xyz)
				float dotrR = (x*cxR+y*cyR+z*czR) / r;
				float dotrI = (x*cxI+y*cyI+z*czI) / r;
				mVelocityX0[ idxR ] = f * ( cxR - x*dotrR );
				mVelocityX0[ idxI ] = f * ( cxI - x*dotrI );
				mVelocityY0[ idxR ] = f * ( cyR - y*dotrR );
				mVelocityY0[ idxI ] = f * ( cyI - y*dotrI );
				mVelocityZ0[ idxR ] = f * ( czR - z*dotrR );
				mVelocityZ0[ idxI ] = f * ( czI - z*dotrI );
			}
		}	 
	}

	// transform to real domain
	fftC2R( mVelocityX0 );
	fftC2R( mVelocityY0 );
	fftC2R( mVelocityZ0 );

	// normalize (as FFTW leaves them unnormalized) and damp
	float f = (1.0f - mDamping*mDeltaTime) / (mTotalSize);
	for( i=0; i<mSize; ++i ) {
		for( j=0; j<mSize; ++j ) {
			for( k=0 ; k<mSize; k++ ) {
				int idx = i + mSize*j + mSize*mSize*k;
				int idx0 = i + (mSize+2)*j + mSize*(mSize+2)*k;
				mVelocityX[ idx ] = f * mVelocityX0[ idx0 ];
				mVelocityY[ idx ] = f * mVelocityY0[ idx0 ];
				mVelocityZ[ idx ] = f * mVelocityZ0[ idx0 ];
			}
		}
	}
}

void CFluid::solveTemperature()
{
	int i, j, k;
	
	//
	// self-advection (diffuse temperatures)

	float x, y, z;
	float invSize = 1.0f / mSize;
	float invHalfSize = invSize * 0.5f;
	for( i = 0, x = invHalfSize; i < mSize; ++i, x += invSize ) {
		for( j = 0, y = invHalfSize; j < mSize; ++j, y += invSize ) {
			for( k = 0, z = invHalfSize; k < mSize; ++k, z += invSize ) {
				int idxIJK = i + mSize*j + mSize*mSize*k;
				float x0 = mSize * ( x - mDeltaTime * mVelocityX[ idxIJK ] ) - 0.5f;
				float y0 = mSize * ( y - mDeltaTime * mVelocityY[ idxIJK ] ) - 0.5f;
				float z0 = mSize * ( z - mDeltaTime * mVelocityZ[ idxIJK ] ) - 0.5f;
				interpolatePrepare( x0, y0, z0 );
				mTemperature[idxIJK] = interpolate( mTemperature0 );
			}
		}
	}

	//
	// damp temperature

	float f = 1.0f / (1.0f + mTemperatureDissipation*mDeltaTime);
	
	for( i = 0; i < mTotalSize; ++i ) {
		mTemperature0[i] = mTemperature[i] * f;
	}
}
