
#ifndef __FLUID_H
#define __FLUID_H

#include <rfftw.h>
#include <d3dx8math.h>


class CFluid {
public:
	CFluid( int size/*, bool cylinder*/ );
	~CFluid();
	
	int		getSize() const { return mSize; }
	int		getLayerSize() const { return mLayerSize; }
	int		getTotalSize() const { return mTotalSize; }
	
	float	getDeltaTime() const { return mDeltaTime; }
	float	getViscosity() const { return mViscosity; }
	float	getDamping() const { return mDamping; }
	float	getTemperatureDissipation() const { return mTemperatureDissipation; }
	
	void	setDeltaTime( float dt ) { mDeltaTime = dt; }
	void	setViscosity( float v ) { mViscosity = v; }
	void	setDamping( float d ) { mDamping = d; }
	void	setTemperatureDissipation( float d ) { mTemperatureDissipation = d; }
	
	void	solveVelocity();
	void	solveTemperature();

	void	resetVelocity0();

	int		getIndex( int ix, int iy, int iz ) const { return ix + mSize*iy + mLayerSize*iz; }
	float* getVelocityX() { return mVelocityX; }
	float* getVelocityY() { return mVelocityY; }
	float* getVelocityZ() { return mVelocityZ; }
	float* getTemperature() { return mTemperature; }
	const float* getVelocityX() const { return mVelocityX; }
	const float* getVelocityY() const { return mVelocityY; }
	const float* getVelocityZ() const { return mVelocityZ; }
	const float* getTemperature() const { return mTemperature; }
	float* getVelocityX0() { return mVelocityX0; }
	float* getVelocityY0() { return mVelocityY0; }
	float* getVelocityZ0() { return mVelocityZ0; }
	float* getTemperature0() { return mTemperature0; }
	const float* getVelocityX0() const { return mVelocityX0; }
	const float* getVelocityY0() const { return mVelocityY0; }
	const float* getVelocityZ0() const { return mVelocityZ0; }
	const float* getTemperature0() const { return mTemperature0; }

	struct SInterpolatePrepare {
		int		i0, i1, j0, j1, k0, k1;
		float	fi, fj, fk;
		int		idx[8];
	};
	void	interpolatePrepare( float x, float y, float z ) const;
	float	interpolate( const float* field ) const;
	const SInterpolatePrepare& getInterpolate() const {
		return mInterpolate;
	}
	
private:
	void fftR2C( float* p ) {
		rfftwnd_one_real_to_complex( mPlanR2C,(float*)p,(fftw_complex*)p );
	}
	void fftC2R( float* p ) {
		rfftwnd_one_complex_to_real( mPlanC2R,(fftw_complex*)p,(float*)p );
	}
	static int floori( float x ) {
		return (x >= 0.0f) ? (int)x : -(int)(1.0f-x);
	}
	static int floori( double x ) {
		return (x >= 0.0) ? (int)x : -(int)(1.0-x);
	}
	
private:
	//bool	mCylinder;
	
	int 	mSize;
	int 	mLayerSize; // mSize * mSize
	int 	mTotalSize; // mSize * mSize * mSize
	
	float	mDeltaTime;
	float	mViscosity;
	float	mDamping;
	float	mTemperatureDissipation;
	
	//int*	mInside;

	float*	mVelocityX;
	float*	mVelocityX0;
	float*	mVelocityY;
	float*	mVelocityY0;
	float*	mVelocityZ;
	float*	mVelocityZ0;
	float*	mTemperature;
	float*	mTemperature0;
	
	rfftwnd_plan	mPlanR2C;
	rfftwnd_plan	mPlanC2R;

	mutable SInterpolatePrepare	mInterpolate;
};


#endif
