
#include "WaterSurface.h"
#include "Resources.h"
#include "../Vertices.h"
#include "Fluid.h"
#include "../utils/random.h"
#include <cmath>
#include <d3d8.h>
#include <cassert>

CWaterSurface::CWaterSurface( CResources& resources )
:	mResources(&resources)
{
	resetVelocities();
}

void CWaterSurface::resetVelocities()
{
	ZeroMemory( mVelocities, sizeof(mVelocities) );
}

void CWaterSurface::feedVelocity( float x, float y, const D3DXVECTOR3& v, float deltaTime )
{
	int ix = (int)((x+R_FLUID_RADIUS) / (R_FLUID_RADIUS*2/SIZE) + 0.5f) & (SIZE-1);
	int iy = (int)((y+R_FLUID_RADIUS) / (R_FLUID_RADIUS*2/SIZE) + 0.5f) & (SIZE-1);
	mVelocities[ ix + iy*SIZE ] += v * deltaTime;
}

void CWaterSurface::updateWaterMesh( float deltaTime )
{
	int i;

	// copy surface XY coords from mesh clone in system memory.
	// calculate surface Z coord and normals from velocities in Fluid.

	int nVerts = (int)mResources->mMeshWaterSystem->GetNumVertices();

	IDirect3DVertexBuffer8* vbSystem;
	mResources->mMeshWaterSystem->GetVertexBuffer( &vbSystem );
	SVertexXyz* vSystem;
	vbSystem->Lock( 0, 0, (BYTE**)&vSystem, D3DLOCK_READONLY );

	IDirect3DVertexBuffer8* vbDynamic;
	mResources->mMeshWaterDynamic->GetVertexBuffer( &vbDynamic );
	SVertexXyzNormal* vDynamic;
	vbDynamic->Lock( 0, 0, (BYTE**)&vDynamic, D3DLOCK_DISCARD );
	
	for( i = 0; i < nVerts; ++i ) {
		float px = vSystem->p.x;
		float py = vSystem->p.y;
		int ix = (int)((px+R_FLUID_RADIUS) / (R_FLUID_RADIUS*2/SIZE) + 0.5f) & (SIZE-1);
		int iy = (int)((py+R_FLUID_RADIUS) / (R_FLUID_RADIUS*2/SIZE) + 0.5f) & (SIZE-1);
		int idx = ix + iy*SIZE;
		const D3DXVECTOR3& vel = mVelocities[idx];
		const D3DXVECTOR3& velR = mVelocities[(idx+1)&(LAYER_SIZE-1)];
		const D3DXVECTOR3& velD = mVelocities[(idx+SIZE)&(LAYER_SIZE-1)];
		
		D3DXVECTOR3 vN;
		vN.x = atanf(vel.x-velR.x) + random::randfs( 0.001f );
		vN.y = atanf(vel.y-velD.y) + random::randfs( 0.001f );
		vN.z = 1.0f;
		
		vDynamic->p.x = vSystem->p.x;
		vDynamic->p.y = vSystem->p.y;
		vDynamic->p.z = vSystem->p.z;

		vDynamic->n = vN;

		++vSystem;
		++vDynamic;
	}

	vbSystem->Unlock();
	vbDynamic->Unlock();
	vbSystem->Release();
	vbDynamic->Release();

	for( i = 0; i < LAYER_SIZE; ++i ) {
		mVelocities[i] *= 1.0f / (1.0f + 20.0f * deltaTime);
	}
}

/*
void CWaterSurface::updateWaterMesh()
{
	// copy surface XY coords from mesh clone in system memory.
	// calculate surface Z coord and normals from velocities in Fluid.
	
	int nVerts = (int)mResources->mMeshWaterSystem->GetNumVertices();
	
	IDirect3DVertexBuffer8* vbSystem;
	mResources->mMeshWaterSystem->GetVertexBuffer( &vbSystem );
	SVertexXyz* vSystem;
	vbSystem->Lock( 0, 0, (BYTE**)&vSystem, D3DLOCK_READONLY );
	
	CFluid& fluid = *mResources->mFluid;
	
	const float scale = (fluid.getSize()/2) / R_FLUID_RADIUS;
	const float step = R_FLUID_RADIUS / (fluid.getSize()/2);
	
	IDirect3DVertexBuffer8* vbDynamic;
	mResources->mMeshWaterDynamic->GetVertexBuffer( &vbDynamic );
	SVertexXyzNormal* vDynamic;
	vbDynamic->Lock( 0, 0, (BYTE**)&vDynamic, D3DLOCK_DISCARD );
	
	for( int i = 0; i < nVerts; ++i ) {
		float px = vSystem->p.x * scale;
		float py = vSystem->p.y * scale;
		float pz = vSystem->p.z * scale;
		fluid.interpolatePrepare( px, py, pz );
		float velx = fluid.interpolate( fluid.getVelocityX() );
		float vely = fluid.interpolate( fluid.getVelocityY() );
		float velz = fluid.interpolate( fluid.getVelocityZ() );
		
		
		// ...
		
		fluid.interpolatePrepare( px * 2.6f, py * 2.6f, pz * 0.95f + px*1.7f + py * 2.6f );
		float velx2 = fluid.interpolate( fluid.getVelocityX() ) * 0.8f;
		float vely2 = fluid.interpolate( fluid.getVelocityY() ) * 0.8f;
		
		fluid.interpolatePrepare( px * 6.7f, py * 6.7f, pz * 0.90f + px*2.3f + py * 1.3f );
		float velx3 = fluid.interpolate( fluid.getVelocityX() ) * 0.8f;
		float vely3 = fluid.interpolate( fluid.getVelocityY() ) * 0.8f;
		
		D3DXVECTOR3 vN;
		vN.x = -atanf(velx+velx2+velx3);
		vN.y = -atanf(vely+vely2+velx3);
		vN.z = 0.2f;
		
		vDynamic->p.x = vSystem->p.x;
		vDynamic->p.y = vSystem->p.y;
		vDynamic->p.z = vSystem->p.z;
		
		vDynamic->n = vN;
		
		++vSystem;
		++vDynamic;
	}
	
	vbSystem->Unlock();
	vbDynamic->Unlock();
}
*/
